import java.util.*;
import javax.sdp.*;


//Esta clase implementa el protocolo SDP, el cual es usado por SIP para
//negociar el tipo de sesion que se desea realizar. Para este caso es solo
//una sesion de audio enviado por RTP/IPv4
public class SdpManager {

  SdpFactory mySdpFactory;
  SdpInfo mySdpInfo;
  byte[] mySdpContent;

  public SdpManager() {
    mySdpFactory = SdpFactory.getInstance();
  }

  public byte[] createSdp(SdpInfo sdpinfo) {

    try{
    //Session-level description lines
    Version myVersion = mySdpFactory.createVersion(0);
    long ss=mySdpFactory.getNtpTime(new Date());
    Origin myOrigin = mySdpFactory.createOrigin("-",ss,ss,"IN","IP4",sdpinfo.IpAddress);
    SessionName mySessionName = mySdpFactory.createSessionName("-");
    Connection myConnection = mySdpFactory.createConnection("IN","IP4", sdpinfo.IpAddress);

    //Time description lines
    Time myTime=mySdpFactory.createTime();
    Vector myTimeVector=new Vector();
    myTimeVector.add(myTime);

    //Media description lines
    int[] aaf=new int[1];
    aaf[0]=sdpinfo.aformat;

    //aaf[0]=sdpinfo.getAFormat();
    MediaDescription myAudioDescription = mySdpFactory.createMediaDescription("audio", sdpinfo.aport, 1, "RTP/AVP", aaf);
    Vector myMediaDescriptionVector=new Vector();
    myMediaDescriptionVector.add(myAudioDescription);

    SessionDescription mySdp = mySdpFactory.createSessionDescription();
    mySdp.setVersion(myVersion);
    mySdp.setOrigin(myOrigin);
    mySdp.setSessionName(mySessionName);
    mySdp.setConnection(myConnection);
    mySdp.setTimeDescriptions(myTimeVector);
    mySdp.setMediaDescriptions(myMediaDescriptionVector);
    mySdpContent=mySdp.toString().getBytes();
    }catch(Exception e){
      e.printStackTrace();
    }
    return mySdpContent;
  }

  public SdpInfo getSdp(byte[] content) {
    try{
      String s = new String(content);
      SessionDescription recSdp = mySdpFactory.createSessionDescription(s);
      String myPeerIp=recSdp.getConnection().getAddress();
      String myPeerName=recSdp.getOrigin().getUsername();
      Vector recMediaDescriptionVector=recSdp.getMediaDescriptions(false);  
      MediaDescription myAudioDescription = (MediaDescription) recMediaDescriptionVector.elementAt(0);
      Media myAudio = myAudioDescription.getMedia();
      int myAudioPort = myAudio.getMediaPort();
      Vector audioFormats = myAudio.getMediaFormats(false);
      int myAudioMediaFormat = Integer.parseInt(audioFormats.elementAt(0).toString());
      System.out.println(recMediaDescriptionVector.size());
      mySdpInfo=new SdpInfo();
      mySdpInfo.IpAddress=myPeerIp;
      mySdpInfo.aport=myAudioPort;
      mySdpInfo.aformat=myAudioMediaFormat;
    }catch(Exception e){
      e.printStackTrace();
    }
    return mySdpInfo;
  }
}