import javax.sip.*;
import javax.sip.message.*;
import javax.sip.header.*;
import javax.sip.address.*;
import java.net.*;
import java.util.*;

public class SimpleSIPListener implements SipListener{

private SipFactory mySipFactory;
private SipStack mySipStack;
private ListeningPoint myListeningPoint;
private SipProvider mySipProvider;
private MessageFactory myMessageFactory;
private HeaderFactory myHeaderFactory;
private AddressFactory myAddressFactory;
private Properties myProperties;
private SimpleSIPGUI myGUI;
private ContactHeader myContactHeader;
private ViaHeader myViaHeader;
private Address fromAddress;
private Dialog myDialog;
private ClientTransaction myClientTransaction;
private ServerTransaction myServerTransaction;
private int status;
private String myIP;
private SdpManager mySdpManager;
private VoiceTool myVoiceTool;
private SdpInfo answerInfo;
private SdpInfo offerInfo;

private int myPort;
private int myAudioPort;
private int myAudioCodec;

static final int YES=0;
static final int NO=1;

static final int IDLE=0;
static final int WAIT_PROV=1;
static final int WAIT_FINAL=2;
static final int ESTABLISHED=4;
static final int RINGING=5;
static final int WAIT_ACK=6;

class MyTimerTask extends TimerTask {
        SimpleSIPListener myListener;
        public MyTimerTask (SimpleSIPListener myListener){
                this.myListener=myListener;
              }
        public void run() {
          try{
            Request myBye = myListener.myDialog.createRequest("BYE");
            myBye.addHeader(myListener.myContactHeader);
            myListener.myClientTransaction =
                myListener.mySipProvider.getNewClientTransaction(myBye);
            myListener.myDialog.sendRequest(myListener.myClientTransaction);
          }catch (Exception ex){
            ex.printStackTrace();
        }
    }
    }

  public SimpleSIPListener(Configuration conf,SimpleSIPGUI GUI)  {
    try{
      myGUI = GUI;
      myIP = InetAddress.getLocalHost().getHostAddress();
      myPort = conf.sipPort;
      myAudioPort=conf.audioPort;
      myAudioCodec=conf.audioCodec;
      mySdpManager=new SdpManager();
      myVoiceTool=new VoiceTool();
      answerInfo=new SdpInfo();
      offerInfo=new SdpInfo();
      mySipFactory = SipFactory.getInstance();
      mySipFactory.setPathName("gov.nist");
      myProperties = new Properties();
      myProperties.setProperty("javax.sip.STACK_NAME", "myStack");
//      mySipStack = mySipFactory.createSipStack(myProperties);
      myMessageFactory = mySipFactory.createMessageFactory();
      myHeaderFactory = mySipFactory.createHeaderFactory();
      myAddressFactory = mySipFactory.createAddressFactory();
      myListeningPoint = mySipStack.createListeningPoint(myIP, myPort, "udp");
      mySipProvider = mySipStack.createSipProvider(myListeningPoint);
      mySipProvider.addSipListener(this);
      Address contactAddress = myAddressFactory.createAddress("sip:"+myIP+":"+myPort);
      myContactHeader = myHeaderFactory.createContactHeader(contactAddress);
      fromAddress=myAddressFactory.createAddress(conf.name+ " <sip:"+conf.userID+"@"+myIP+":"+myPort+">");
      status=IDLE;
      myGUI.jLabel5.setText("At "+myIP+", port "+myPort);
      myGUI.showStatus("Estado: Desocupado");
    }catch (Exception e) {
     e.printStackTrace();
    }
  }

  public void setOff(){
    try{
    mySipProvider.removeSipListener(this);
    mySipProvider.removeListeningPoint(myListeningPoint);
    mySipStack.deleteListeningPoint(myListeningPoint);
    mySipStack.deleteSipProvider(mySipProvider);
    myListeningPoint=null;
    mySipProvider=null;
    mySipStack=null;
    myGUI.showStatus("");
    }
    catch(Exception e){}
  }

public void updateConfiguration(Configuration conf) {
  myPort = conf.sipPort;
  myAudioPort=conf.audioPort;
  myAudioCodec=conf.audioCodec;
}

public void processRequest(RequestEvent requestReceivedEvent) {
  Request myRequest=requestReceivedEvent.getRequest();
  String method=myRequest.getMethod();
  myGUI.display("<<< "+myRequest.toString());
  if (!method.equals("CANCEL")) {
  myServerTransaction=requestReceivedEvent.getServerTransaction();
  }

  try{
  switch (status) {
    case IDLE:
        //al llamar
      if (method.equals("INVITE")) {
        if (myServerTransaction == null) {
                myServerTransaction = mySipProvider.getNewServerTransaction(myRequest);
        }
        byte[] cont=(byte[]) myRequest.getContent();
        offerInfo=mySdpManager.getSdp(cont);
        answerInfo.IpAddress=myIP;
        answerInfo.aport=myAudioPort;
        answerInfo.aformat=offerInfo.aformat;
        Response myResponse=myMessageFactory.createResponse(180,myRequest);
        myResponse.addHeader(myContactHeader);
        ToHeader myToHeader = (ToHeader) myResponse.getHeader("To");
        myToHeader.setTag("454326");
        myServerTransaction.sendResponse(myResponse);
        myDialog=myServerTransaction.getDialog();
        myGUI.display(">>> "+myResponse.toString());
        status=RINGING;
        myGUI.showStatus("Estado: Llamando");
      }
     break;
    case ESTABLISHED://llamada establecida

      if (method.equals("BYE")) {//cerrar la llamada
        Response myResponse=myMessageFactory.createResponse(200,myRequest);
        myResponse.addHeader(myContactHeader);
        myServerTransaction.sendResponse(myResponse);
        myGUI.display(">>> "+myResponse.toString());
       myVoiceTool.stopMedia();
        status=IDLE;
        myGUI.showStatus("Estado: Desocupado");
      }
    break;

    case RINGING://esta sonando

      if (method.equals("CANCEL")) {//si el usuario se arrepiente de seguir llamando
        ServerTransaction myCancelServerTransaction=requestReceivedEvent.getServerTransaction();
        Request originalRequest=myServerTransaction.getRequest();
        Response myResponse=myMessageFactory.createResponse(487,originalRequest);
        myServerTransaction.sendResponse(myResponse);
        Response myCancelResponse=myMessageFactory.createResponse(200,myRequest);
        myCancelServerTransaction.sendResponse(myCancelResponse);
        myGUI.display(">>> "+myResponse.toString());
        myGUI.display(">>> "+myCancelResponse.toString());
        status=IDLE;
        myGUI.showStatus("Estado: Desocupado");
      }
      break;

      case WAIT_ACK:
          //el usuario espera pro la confirmacion y la recibe al hacer la llamada OK
        if (method.equals("ACK")) {
        status=ESTABLISHED;
        myGUI.showStatus("Estado: Conectado");
      }
  }
  }catch (Exception e) {
    e.printStackTrace();
  }
}


public void processResponse(ResponseEvent responseReceivedEvent) {
  try{
  Response myResponse=responseReceivedEvent.getResponse();
  myGUI.display("<<< "+myResponse.toString());
  ClientTransaction thisClientTransaction=responseReceivedEvent.getClientTransaction();
  if (!thisClientTransaction.equals(myClientTransaction)) {return;}
  int myStatusCode=myResponse.getStatusCode();
  CSeqHeader originalCSeq=(CSeqHeader) myClientTransaction.getRequest().getHeader(CSeqHeader.NAME);
  long numseq=originalCSeq.getSeqNumber();

switch(status){

  case WAIT_PROV:

    if (myStatusCode<200) {//todavia no es posible hacer la llamada: codigo 180 por ejemplo (esta sonando)
      status=WAIT_FINAL;
      myDialog=thisClientTransaction.getDialog();
      myGUI.showStatus("Estado: Alertando");
    }
    else if (myStatusCode<300) {//establece la llamada: codigo ok 200 por ejemplo
      myDialog=thisClientTransaction.getDialog();
      Request myAck = myDialog.createAck(numseq);
      myAck.addHeader(myContactHeader);
      myDialog.sendAck(myAck);
      myGUI.display(">>> "+myAck.toString());
      status=ESTABLISHED;
      myGUI.showStatus("Estado: Conectado");
      byte[] cont=(byte[]) myResponse.getContent();
      answerInfo=mySdpManager.getSdp(cont);
      myVoiceTool.startMedia(answerInfo.IpAddress,answerInfo.aport,offerInfo.aport,answerInfo.aformat);
    }
    else {
      status=IDLE;//pasa a no hacer nada en particular, solo esta desogupado
      Request myAck = myDialog.createAck(numseq);
      myAck.addHeader(myContactHeader);
      myDialog.sendAck(myAck);
      myGUI.display(">>> "+myAck.toString());
      myGUI.showStatus("Estado: Desocupado");
    }
    break;

  case WAIT_FINAL:
    if (myStatusCode<200) {// alertando
      status=WAIT_FINAL;
      myDialog=thisClientTransaction.getDialog();
      myGUI.showStatus("Estado: Alertando");
    }
    else if (myStatusCode<300) {
      status=ESTABLISHED;
      myDialog=thisClientTransaction.getDialog();
      Request myAck = myDialog.createAck(numseq);
      myAck.addHeader(myContactHeader);
      myDialog.sendAck(myAck);
      myGUI.display(">>> "+myAck.toString());
      myGUI.showStatus("Estado: Conectado");
      byte[] cont=(byte[]) myResponse.getContent();
      answerInfo=mySdpManager.getSdp(cont);
      myVoiceTool.startMedia(answerInfo.IpAddress,answerInfo.aport,offerInfo.aport,answerInfo.aformat);
      System.out.println("EL LLAMANTE ESCUCHA EN"+offerInfo.aport);
    }
    else {
      status=IDLE;
      myGUI.showStatus("Estado: Desocupado");
    }
    break;
}
  }catch(Exception excep){
    excep.printStackTrace();
  }
}

  public void processTimeout(TimeoutEvent timeoutEvent) {
  }

  public void processTransactionTerminated(TransactionTerminatedEvent tevent) {
  }

  public void processDialogTerminated(DialogTerminatedEvent tevent) {
  }

  public void processIOException(IOExceptionEvent tevent) {
   }

public void userInput(int type, String destination){
     try {
       switch (status) {
         case IDLE:
           if (type == YES) {
             Address toAddress = myAddressFactory.createAddress(destination);
             ToHeader myToHeader = myHeaderFactory.createToHeader(toAddress, null);
             FromHeader myFromHeader = myHeaderFactory.createFromHeader(
                 fromAddress, "56438");
             myViaHeader = myHeaderFactory.createViaHeader(myIP, myPort,"udp", null);
             ArrayList myViaHeaders = new ArrayList();
             myViaHeaders.add(myViaHeader);
             MaxForwardsHeader myMaxForwardsHeader = myHeaderFactory.
                 createMaxForwardsHeader(70);
             CSeqHeader myCSeqHeader = myHeaderFactory.createCSeqHeader(1L,
                 "INVITE");
             CallIdHeader myCallIDHeader = mySipProvider.getNewCallId();
             javax.sip.address.URI myRequestURI = toAddress.getURI();
             Request myRequest = myMessageFactory.createRequest(myRequestURI,
                 "INVITE",
                 myCallIDHeader, myCSeqHeader, myFromHeader, myToHeader,
                 myViaHeaders, myMaxForwardsHeader);
             myRequest.addHeader(myContactHeader);
             offerInfo=new SdpInfo();
             offerInfo.IpAddress=myIP;
             offerInfo.aport=myAudioPort;
             offerInfo.aformat=myAudioCodec;
             ContentTypeHeader contentTypeHeader=myHeaderFactory.createContentTypeHeader("application","sdp");
             byte[] content=mySdpManager.createSdp(offerInfo);
             myRequest.setContent(content,contentTypeHeader);
             myClientTransaction = mySipProvider.getNewClientTransaction(myRequest);
             String bid=myClientTransaction.getBranchId();
             myClientTransaction.sendRequest();
             myDialog = myClientTransaction.getDialog();
             myGUI.display(">>> " + myRequest.toString());
             status = WAIT_PROV;
             myGUI.showStatus("Estado: Esperando PROV");
             break;
           }

         case WAIT_FINAL:
           if (type == NO) {
             Request myCancelRequest = myClientTransaction.createCancel();
             ClientTransaction myCancelClientTransaction = mySipProvider.
                 getNewClientTransaction(myCancelRequest);
             myCancelClientTransaction.sendRequest();
             myGUI.display(">>> " + myCancelRequest.toString());
             status = IDLE;
             myGUI.showStatus("Estado: Desocupado");
             break;
           }

         case ESTABLISHED:
           if (type == NO) {
             Request myBye = myDialog.createRequest("BYE");
             myBye.addHeader(myContactHeader);
             myClientTransaction= mySipProvider.getNewClientTransaction(myBye);
             myDialog.sendRequest(myClientTransaction);
             myGUI.display(">>> " + myBye.toString());
             myVoiceTool.stopMedia();
             status = IDLE;
             myGUI.showStatus("Estado: Desocupado");
             break;
           }

         case RINGING:
           if (type == NO) {
             Request originalRequest = myServerTransaction.getRequest();
             Response myResponse = myMessageFactory.createResponse(486,
                 originalRequest);
             myServerTransaction.sendResponse(myResponse);
             myGUI.display(">>> " + myResponse.toString());
             status = IDLE;
             myGUI.showStatus("Estado: Desocupado");
             break;
           }
           else if (type == YES) {
             Request originalRequest = myServerTransaction.getRequest();
             Response myResponse = myMessageFactory.createResponse(200,
                 originalRequest);
             ToHeader myToHeader = (ToHeader) myResponse.getHeader("To");
             myToHeader.setTag("454326");
             myResponse.addHeader(myContactHeader);
             ContentTypeHeader contentTypeHeader=myHeaderFactory.createContentTypeHeader("application","sdp");
             byte[] content=mySdpManager.createSdp(answerInfo);
             myResponse.setContent(content,contentTypeHeader);
             myVoiceTool.startMedia(offerInfo.IpAddress,offerInfo.aport,answerInfo.aport,offerInfo.aformat);
             myServerTransaction.sendResponse(myResponse);
             myDialog = myServerTransaction.getDialog();
             new Timer().schedule(new MyTimerTask(this),500000);
             myGUI.display(">>> " + myResponse.toString());
             status = WAIT_ACK;
             myGUI.showStatus("Estado: Esperando ACK");
             break;
           }
       }
     }
     catch (Exception e){
     e.printStackTrace();
   }
   }
}